@@ -1,14 +1,18 @@
 from __future__ import annotations
 
+import typing as t
+
 from sqlglot import exp
 from sqlglot.dialects.dialect import (
     approx_count_distinct_sql,
-    build_timestamp_trunc,
+    property_sql,
     rename_func,
     time_format,
     unit_to_str,
 )
 from sqlglot.dialects.mysql import MySQL
+from sqlglot.helper import seq_get
+from sqlglot.tokens import TokenType
 
 
 def _lag_lead_sql(self, expression: exp.Lag | exp.Lead) -> str:
@@ -20,6 +24,22 @@ def _lag_lead_sql(self, expression: exp.Lag | exp.Lead) -> str:
     )
 
 
+# Accept both DATE_TRUNC(datetime, unit) and DATE_TRUNC(unit, datetime)
+def _build_date_trunc(args: t.List[exp.Expression]) -> exp.Expression:
+    a0, a1 = seq_get(args, 0), seq_get(args, 1)
+
+    def _is_unit_like(e: exp.Expression | None) -> bool:
+        if not (isinstance(e, exp.Literal) and e.is_string):
+            return False
+        text = e.this
+        return not any(ch.isdigit() for ch in text)
+
+    # Determine which argument is the unit
+    unit, this = (a0, a1) if _is_unit_like(a0) else (a1, a0)
+
+    return exp.TimestampTrunc(this=this, unit=unit)
+
+
 class Doris(MySQL):
     DATE_FORMAT = "'yyyy-MM-dd'"
     DATEINT_FORMAT = "'yyyyMMdd'"
@@ -29,7 +49,7 @@ class Doris(MySQL):
         FUNCTIONS = {
             **MySQL.Parser.FUNCTIONS,
             "COLLECT_SET": exp.ArrayUniqueAgg.from_arg_list,
-            "DATE_TRUNC": build_timestamp_trunc,
+            "DATE_TRUNC": _build_date_trunc,
             "MONTHS_ADD": exp.AddMonths.from_arg_list,
             "REGEXP": exp.RegexpLike.from_arg_list,
             "TO_DATE": exp.TsOrDsToDate.from_arg_list,
@@ -38,9 +58,81 @@ class Doris(MySQL):
         FUNCTION_PARSERS = MySQL.Parser.FUNCTION_PARSERS.copy()
         FUNCTION_PARSERS.pop("GROUP_CONCAT")
 
+        NO_PAREN_FUNCTIONS = MySQL.Parser.NO_PAREN_FUNCTIONS.copy()
+        NO_PAREN_FUNCTIONS.pop(TokenType.CURRENT_DATE)
+
+        PROPERTY_PARSERS = {
+            **MySQL.Parser.PROPERTY_PARSERS,
+            "PROPERTIES": lambda self: self._parse_wrapped_properties(),
+            "UNIQUE": lambda self: self._parse_composite_key_property(exp.UniqueKeyProperty),
+            "PARTITION BY": lambda self: self._parse_partition_by_opt_range(),
+        }
+
+        def _parse_partitioning_granularity_dynamic(self) -> exp.PartitionByRangePropertyDynamic:
+            self._match_text_seq("FROM")
+            start = self._parse_wrapped(self._parse_string)
+            self._match_text_seq("TO")
+            end = self._parse_wrapped(self._parse_string)
+            self._match_text_seq("INTERVAL")
+            number = self._parse_number()
+            unit = self._parse_var(any_token=True)
+            every = self.expression(exp.Interval, this=number, unit=unit)
+            return self.expression(
+                exp.PartitionByRangePropertyDynamic, start=start, end=end, every=every
+            )
+
+        def _parse_partition_definition(self) -> exp.Partition:
+            self._match_text_seq("PARTITION")
+
+            name = self._parse_id_var()
+            self._match_text_seq("VALUES")
+
+            if self._match_text_seq("LESS", "THAN"):
+                values = self._parse_wrapped_csv(self._parse_expression)
+                if len(values) == 1 and values[0].name.upper() == "MAXVALUE":
+                    values = [exp.var("MAXVALUE")]
+
+                part_range = self.expression(exp.PartitionRange, this=name, expressions=values)
+                return self.expression(exp.Partition, expressions=[part_range])
+
+            self._match(TokenType.L_BRACKET)
+            values = self._parse_csv(lambda: self._parse_wrapped_csv(self._parse_expression))
+
+            self._match(TokenType.R_BRACKET)
+            self._match(TokenType.R_PAREN)
+
+            part_range = self.expression(exp.PartitionRange, this=name, expressions=values)
+            return self.expression(exp.Partition, expressions=[part_range])
+
+        def _parse_partition_by_opt_range(
+            self,
+        ) -> exp.PartitionedByProperty | exp.PartitionByRangeProperty:
+            if not self._match_text_seq("RANGE"):
+                return super()._parse_partitioned_by()
+
+            partition_expressions = self._parse_wrapped_id_vars()
+            self._match_l_paren()
+
+            if self._match_text_seq("FROM", advance=False):
+                create_expressions = self._parse_csv(self._parse_partitioning_granularity_dynamic)
+            elif self._match_text_seq("PARTITION", advance=False):
+                create_expressions = self._parse_csv(self._parse_partition_definition)
+            else:
+                create_expressions = None
+
+            self._match_r_paren()
+
+            return self.expression(
+                exp.PartitionByRangeProperty,
+                partition_expressions=partition_expressions,
+                create_expressions=create_expressions,
+            )
+
     class Generator(MySQL.Generator):
         LAST_DAY_SUPPORTS_DATE_PART = False
         VARCHAR_REQUIRES_SIZE = False
+        WITH_PROPERTIES_PREFIX = "PROPERTIES"
+        RENAME_TABLE_WITH_DB = False
 
         TYPE_MAPPING = {
             **MySQL.Generator.TYPE_MAPPING,
@@ -49,6 +141,13 @@ class Doris(MySQL):
             exp.DataType.Type.TIMESTAMPTZ: "DATETIME",
         }
 
+        PROPERTIES_LOCATION = {
+            **MySQL.Generator.PROPERTIES_LOCATION,
+            exp.UniqueKeyProperty: exp.Properties.Location.POST_SCHEMA,
+            exp.PartitionByRangeProperty: exp.Properties.Location.POST_SCHEMA,
+            exp.PartitionedByProperty: exp.Properties.Location.POST_SCHEMA,
+        }
+
         CAST_MAPPING = {}
         TIMESTAMP_FUNC_TYPES = set()
 
@@ -59,7 +158,9 @@ class Doris(MySQL):
             exp.ArgMax: rename_func("MAX_BY"),
             exp.ArgMin: rename_func("MIN_BY"),
             exp.ArrayAgg: rename_func("COLLECT_LIST"),
+            exp.ArrayToString: rename_func("ARRAY_JOIN"),
             exp.ArrayUniqueAgg: rename_func("COLLECT_SET"),
+            exp.CurrentDate: lambda self, _: self.func("CURRENT_DATE"),
             exp.CurrentTimestamp: lambda self, _: self.func("NOW"),
             exp.DateTrunc: lambda self, e: self.func("DATE_TRUNC", e.this, unit_to_str(e)),
             exp.GroupConcat: lambda self, e: self.func(
@@ -69,8 +170,10 @@ class Doris(MySQL):
             exp.Lag: _lag_lead_sql,
             exp.Lead: _lag_lead_sql,
             exp.Map: rename_func("ARRAY_MAP"),
+            exp.Property: property_sql,
             exp.RegexpLike: rename_func("REGEXP"),
             exp.RegexpSplit: rename_func("SPLIT_BY_STRING"),
+            exp.SchemaCommentProperty: lambda self, e: self.naked_property(e),
             exp.Split: rename_func("SPLIT_BY_STRING"),
             exp.StringToArray: rename_func("SPLIT_BY_STRING"),
             exp.StrToUnix: lambda self, e: self.func("UNIX_TIMESTAMP", e.this, self.format_time(e)),
@@ -558,3 +661,66 @@ class Doris(MySQL):
             "xor",
             "year",
         }
+
+        def partition_sql(self, expression: exp.Partition) -> str:
+            parent = expression.parent
+            if isinstance(parent, exp.PartitionByRangeProperty):
+                return ", ".join(self.sql(e) for e in expression.expressions)
+            return super().partition_sql(expression)
+
+        def partitionrange_sql(self, expression: exp.PartitionRange) -> str:
+            name = self.sql(expression, "this")
+            values = expression.expressions
+
+            if len(values) != 1:
+                # Multiple values: use VALUES [ ... )
+                if values and isinstance(values[0], list):
+                    values_sql = ", ".join(
+                        f"({', '.join(self.sql(v) for v in inner)})" for inner in values
+                    )
+                else:
+                    values_sql = ", ".join(f"({self.sql(v)})" for v in values)
+
+                return f"PARTITION {name} VALUES [{values_sql})"
+
+            return f"PARTITION {name} VALUES LESS THAN ({self.sql(values[0])})"
+
+        def partitionbyrangepropertydynamic_sql(self, expression):
+            # Generates: FROM ("start") TO ("end") INTERVAL N UNIT
+            start = self.sql(expression, "start")
+            end = self.sql(expression, "end")
+            every = expression.args.get("every")
+
+            if every:
+                number = self.sql(every, "this")
+                interval = f"INTERVAL {number} {self.sql(every, 'unit')}"
+            else:
+                interval = ""
+
+            return f"FROM ({start}) TO ({end}) {interval}"
+
+        def partitionbyrangeproperty_sql(self, expression):
+            partition_expressions = ", ".join(
+                self.sql(e) for e in expression.args.get("partition_expressions") or []
+            )
+            create_expressions = expression.args.get("create_expressions") or []
+            # Handle both static and dynamic partition definitions
+            create_sql = ", ".join(self.sql(e) for e in create_expressions)
+            return f"PARTITION BY RANGE ({partition_expressions}) ({create_sql})"
+
+        def partitionedbyproperty_sql(self, expression: exp.PartitionedByProperty) -> str:
+            node = expression.this
+            if isinstance(node, exp.Schema):
+                parts = ", ".join(self.sql(e) for e in node.expressions)
+                return f"PARTITION BY ({parts})"
+            return f"PARTITION BY ({self.sql(node)})"
+
+        def table_sql(self, expression: exp.Table, sep: str = " AS ") -> str:
+            """Override table_sql to avoid AS keyword in UPDATE and DELETE statements."""
+            ancestor = expression.find_ancestor(exp.Update, exp.Delete, exp.Select)
+            if not isinstance(ancestor, exp.Select):
+                sep = " "
+            return super().table_sql(expression, sep=sep)
+
+        def alterrename_sql(self, expression: exp.AlterRename, include_to: bool = True) -> str:
+            return super().alterrename_sql(expression, include_to=False)
