@@ -6,9 +6,16 @@ class TestStarrocks(Validator):
     dialect = "starrocks"
 
     def test_starrocks(self):
+        self.validate_identity("SELECT ARRAY_JOIN([1, 3, 5, NULL], '_', 'NULL')")
+        self.validate_identity("SELECT ARRAY_JOIN([1, 3, 5, NULL], '_')")
         self.validate_identity("ALTER TABLE a SWAP WITH b")
+        self.validate_identity("SELECT ARRAY_AGG(a) FROM x")
+        self.validate_identity("SELECT ST_POINT(10, 20)")
+        self.validate_identity("SELECT ST_DISTANCE_SPHERE(10.1, 20.2, 30.3, 40.4)")
 
     def test_ddl(self):
+        self.validate_identity("CREATE TABLE t (c INT) COMMENT 'c'")
+
         ddl_sqls = [
             "DISTRIBUTED BY HASH (col1) BUCKETS 1",
             "DISTRIBUTED BY HASH (col1)",
@@ -36,6 +43,9 @@ class TestStarrocks(Validator):
         self.validate_identity(
             "CREATE TABLE foo (col1 LARGEINT) DISTRIBUTED BY HASH (col1) BUCKETS 1"
         )
+        self.validate_identity(
+            "CREATE VIEW foo (foo_col1) SECURITY NONE AS SELECT bar_col1 FROM bar"
+        )
 
     def test_identity(self):
         self.validate_identity("SELECT CAST(`a`.`b` AS INT) FROM foo")
